/*******************************************************************************
 *
 * Filename : tripapi_defs.h
 * Copyright: Smaser AG
 * Purpose  : Macros and constants for the TRIP API
 *
 */

#ifndef __TRIPAPI_DEFS_H__
#define __TRIPAPI_DEFS_H__

/* Length of CCL commands when using the legacy DAO mode */
#define MAXORDERLEN     400

/* Manifest constants for TdbBeginTdbs() / TdbEndTdbs() */
#define NO_RESTART              0
#define DISCARD_SESSION         0
#define START_NEW               (char)0     /* Obsolete */
#define ALLOW_RESTART           1
#define SAVE_SESSION            1
#define RESTART                 (char)1     /* Obsolete */

/* Manifest constants for window types used in TdbDefineWindow() */
#define CURRENT_WINDOW          0
#define SHOW_WINDOW             1
#define EXPAND_WINDOW           2
#define DISPLAY_WINDOW          3
#define HISTORY_WINDOW          4
#define SYSINFO_WINDOW          5
#define PRINT_WINDOW            6

/* TdbDaoOrder() completion codes for end-user programs */
#define DAO_SELECT              2
#define DAO_FIND                3
#define DAO_SHOW                4
#define DAO_MORE                5
#define DAO_BASE                6
#define DAO_CALL                7
#define DAO_DISPLAY             8
#define DAO_IMPORT              9
#define DAO_EXPORT              10
#define DAO_MACRO               11
#define DAO_STOP                12
#define DAO_DEFINE              13
#define DAO_LIST                14
#define DAO_NEXT                15
#define DAO_PREV                15
#define DAO_HELP                16
#define DAO_EXPAND              17
#define DAO_PRINT               18
#define DAO_RUN                 19
#define DAO_SAVE                20
#define DAO_DELETE              21
#define DAO_TRACE               22
#define DAO_CONTINUE            23
#define DAO_STATUS              24
#define DAO_PRINT_LOCAL         25
#define DAO_EDIT                26
#define DAO_INSERT              27
#define DAO_UPDATE              28
#define DAO_BACK                29
#define DAO_TOP                 30
#define DAO_MEASURE             31
#define DAO_FREQ                32
#define DAO_INDEX               33
#define DAO_CLOSE               35
#define DAO_FUZZ                39
#define DAO_LOAD                42
#define DAO_RENUM               43
#define DAO_FIND_SAVE           44

/* TdbDaoOrder() completion codes used by TRIPclassic / TRIPclient */
#define DAO_CUT                 1
#define DAO_CONT_EDIT           34
#define DAO_HIDE                36
#define DAO_REVEAL              37
#define DAO_CONT_SHOW           38
#define DAO_SFORM               40
#define DAO_LEAVE               41

/* TdbExecuteCcl() completion codes for end-user programs */
#define CMD_ERROR               0
#define CMD_CUT                 1
#define CMD_SELECT              2
#define CMD_FIND                3
#define CMD_SHOW                4
#define CMD_MORE                5
#define CMD_BASE                6
#define CMD_CALL                7
#define CMD_DISPLAY             8
#define CMD_IMPORT              9
#define CMD_EXPORT              10
#define CMD_MACRO               11
#define CMD_STOP                12
#define CMD_DEFINE              13
#define CMD_LIST                14
#define CMD_NEXT                15
#define CMD_PREV                15
#define CMD_HELP                16
#define CMD_EXPAND              17
#define CMD_PRINT               18
#define CMD_RUN                 19
#define CMD_SAVE                20
#define CMD_DELETE              21
#define CMD_TRACE               22
#define CMD_CONTINUE            23
#define CMD_STATUS              24
#define CMD_PRINT_LOCAL         25
#define CMD_EDIT                26
#define CMD_INSERT              27
#define CMD_UPDATE              28
#define CMD_BACK                29
#define CMD_TOP                 30
#define CMD_MEASURE             31
#define CMD_FREQ                32
#define CMD_INDEX               33
#define CMD_CONT_EDIT           34
#define CMD_CLOSE               35
#define CMD_HIDE                36
#define CMD_REVEAL              37
#define CMD_CONT_SHOW           38
#define CMD_FUZZ                39
#define CMD_SFORM               40
#define CMD_LEAVE               41
#define CMD_LOAD                42
#define CMD_RENUM               43
#define CMD_FIND_SAVE           44
#define CMD_DELETE_SEARCH       45

/* Manifest constants for use in TdbStartTrip(), TdbShellInit(), TdbShellModifyCcl(), etc */
#define LANGUAGE_DEFAULT        0   /* Doesn't read dialect file  */
#define LANGUAGE_ENGLISH        9   /* Reads TDBS$SYS:ENGLISH.CCL */
#define LANGUAGE_SWEDISH        10  /* ..  TDBS$SYS:SWEDISH.CCL   */
#define LANGUAGE_CHINESE        11  /* ..  TDBS$SYS:CHINESE.CCL   */
#define LANGUAGE_GERMAN         12  /* ..  TDBS$SYS:GERMAN.CCL    */
#define LANGUAGE_FRENCH         13  /* ..  TDBS$SYS:FRENCH.CCL    */
#define LANGUAGE_FINNISH        14  /* ..  TDBS$SYS:FINNISH.CCL   */
#define LANGUAGE_NORWEGIAN      15  /* ..  TDBS$SYS:NORWEGIAN.CCL */
#define LANGUAGE_JAPANESE       16  /* ..  TDBS$SYS:JAPANESE.CCL  */

/* Manifest character set constants for use in TdbStartTrip and TdbSessionInfo. */
#define CHSET_NONE               0
#define CHSET_LA1                1
#define CHSET_LA2                2
#define CHSET_LA3                3
#define CHSET_UTF8               16
#define CHSET_CHI                17
#define CHSET_GBK                18
#define CHSET_EUC                19
#define CHSET_SJIS               20

/* Macro for converting a character set constant value into a value suitable
 * for the TdbStartTrip mode parameter.
 */
#define CHARSET_MODE(charset) (charset << 8)

/* Manifest constants for use in TdbShowControl() for output of CONTROL file  */
#define DATABASE_LIST           1   /* Show BASe LIST */
#define USER_LIST               2   /* Show USer */
#define DATABASE_STATUS         3   /* Show BASe R= */
#define PROCEDURE_LIST          4   /* Show PRoc */
#define DATABASE_ACCESS         5   /* Show BASe ACCess */
#define OUTPUT_FORMAT_LIST      6   /* Show Format */
#define ENTRY_FORM_LIST         7   /* Show EFOrm */
#define SEARCH_FORM_LIST        8   /* Show SFOrm */
#define THESAURUS_STATUS        9   /* STatus Xxx */
#define GROUP_LIST              10  /* Show Group */
#define CURRENT_STATUS          11  /* Status */
#define USERS_PER_GROUP         14
#define GROUPS_PER_USER         15
#define EXTENDED_USER_LIST      17  /* Show USer */
#define DATABASE_LIST_NOBAF     18  /* Show BASe LIST, info from BIF not BAF */

/* Manifest constants for use in TdbSetBase() access modes */
#define MODE_BIT_READ_ONLY      1
#define MODE_BIT_READ_WRITE     2
#define MODE_BIT_DELETE_CHECK   4
#define MODE_BIT_COPY           8
#define MODE_BIT_EDIT_PREP      16
#define MODE_BIT_CHANGE_EFO     32
#define MODE_BIT_REVERT_EFO     64

#define MODE_READ               ( MODE_BIT_READ_ONLY )
#define MODE_WRITE              ( MODE_BIT_READ_WRITE | MODE_BIT_READ_ONLY )
#define MODE_DELETE             ( MODE_BIT_DELETE_CHECK | MODE_BIT_READ_WRITE )
#define MODE_COPY               ( MODE_BIT_COPY | MODE_BIT_READ_WRITE )
#define MODE_EDIT               ( MODE_BIT_EDIT_PREP | MODE_BIT_READ_WRITE )
#define MODE_NEW_EFO            ( MODE_BIT_CHANGE_EFO | MODE_BIT_EDIT_PREP | MODE_BIT_READ_WRITE )
#define MODE_OLD_EFO            ( MODE_BIT_REVERT_EFO | MODE_BIT_EDIT_PREP | MODE_BIT_READ_WRITE )


/* Manifest constants for use in TdbCheckBase(), CheckDbCluster(), CheckThes() */
#define CHECK_READ              1
#define CHECK_UPDATE            2
#define CHECK_OWNERSHIP         3
#define CHECK_DELETE            4

/* Manifest constants for use in TdbBaffitMode() / TedCheckMode() */
#define ADD_MODE                1
#define MODIFY_MODE             2
#define DELETE_MODE             3

/* Manifest constants for use in TdbBatchLoadIndex() */
#define LOADIX                  1
#define INDEX                   2
#define LOAD                    3

/* Manifest constants for use with TdbBreak() */
#define BREAK_NONE              0
#define BREAK_USER              1

/* Manifest constants for use with TdbShellType() / TdbCheckField() (arg. type) */
#define FIELD_TEXT              1
#define TEXT_FIELD              1       /* Deprecated */
#define FIELD_PHRASE            3
#define PHRASE_FIELD            3       /* Deprecated */
#define FIELD_INTEGER           9
#define INTEGER_FIELD           9       /* Deprecated */
#define FIELD_NUMBER            10
#define NUMBER_FIELD            10      /* Deprecated */
#define FIELD_DATE              11
#define DATE_FIELD              11      /* Deprecated */
#define FIELD_TIME              12
#define TIME_FIELD              12      /* Deprecated */
#define FIELD_STRING            14
#define STRING_FIELD            14      /* Deprecated */
//#define FIELD_LONG              16

/* Manifest constants for return codes from TdbCheckPartName() */
#define PART_EXIST              16034
#define PART_NOT_EXIST          16002

/* Manifest constants for return codes from TdbCheckRecordName() */
#define RECORD_EXIST            7074
#define RECORD_NOT_EXIST        2882

/* Manifest constants for use with TdbCheckSelectRecord() */
#define NO_SELECTION_MADE       0
#define SELECTION_MADE          1

/* Manifest constants for use with TdbCheckSymbol() classes */
#define CHECK_BASENAME          2
#define CHECK_VIEWNAME          3
#define CHECK_TEXT              4
#define CHECK_PHRASE            5
#define CHECK_NUMBER            6
#define CHECK_DATE              7
#define CHECK_TIME              8
#define CHECK_ALL               9
#define CHECK_FIELDS            10
#define CHECK_MAPS              14

/* Manifest constants for use with TdbCreateTimeStamp() / TdbGetTimeStamp() */
#define DATE_AND_TIME           0
#define DATE_ONLY               1

/* Manifest constants for use with TdbCurrentForm() */
#define CURRENT_OUTPUT          1
#define CURRENT_ENTRY           2

/* Manifest constants for use with TdbGetDisplay() */
#define DISPLAY_NORMAL          1
#define DISPLAY_THES            2
#define DISPLAY_INDIRECT        3

/* Manifest constants for use with TdbNextWindow() */
#define FIELD_LEVEL             0
#define PARAGRAPH_LEVEL         1
#define ALL_FIELDS              0
#define TEXT_FIELDS             1
#define TEPH_FIELDS             2
#define PHRASE_FIELDS           3

/* Manifest constants for use with TdbProcessMode() */
#define INTERACTIVE_MODE        1
#define TTY_MODE                2
#define BATCH_MODE              3

/* Manifest constants for use with TedCheckMode() */
#define RECORD_MODE             1
#define PART_MODE               2        /* not yet implemented */
#define FIELD_MODE              3        /* not yet implemented */

/* Manifest constants for use with TdbGetBlobBlock() / TdbPutBlobBlock() */
#define NO_SUCH_BLOCK           0
#define LAST_BLOCK              1
#define OK_BLOCK                3
#define NO_SUCH_CURSOR          14658
#define NO_SUCH_BLOB            21634

/* A generic lower-limit constant for TRIPapi return codes which may be
 *  interpreted by TdbMessage()
 */
#ifndef MSG_HAS_TEXT
#   define MSG_HAS_TEXT         32
#endif

/* Manifest constants for TdbMessage() */

#ifndef MSG_SET_ERROR
#define MSG_SET_ERROR          -1
#endif

#ifndef MSG_SET_INFO
#define MSG_SET_INFO           -2
#endif

/* Manifest constants for use with TdbImport() and TdbExport() */
#define IEOPT_FILENAME          1
#define IEOPT_FILEPTR           2
#define IEOPT_MEMORY            4
#define EXPORT_BLOCK            16
#define EXPORT_ALLOC            32
#define EXPORT_MKFILE           64
#define EXPORT_FILEAPP          256
#define EXPORT_FILETRUNC        512

/* TRIPxml-specific constants for the filter_options
 * parameter in the filter_data structure.
 */
#define FOXML_NEWREC            1
#define FOXML_REPLACE           2
#define FOXML_VALIDATE          4
#define FOXML_GETBYID           8
#define FOXML_REMAKE            16
#define FOXML_HILIGHT           32
#define FOXML_NOBLOB            64
#define FOXML_STREAM            256 ///< for server from client download!

/* Manifest constants for use with TdbBaseInfo() */
#define BASEINFO_XML            1
#define BASEINFO_CHARFOLD       2
#define BASEINFO_RNAMEFLD       3
#define BASEINFO_MAXFLD         4
#define BASEINFO_PNAMEFLD       5
#define BASEINFO_LANGUAGE       6
#define BASEINFO_CHARSET        7
#define BASEINFO_RNUMFLD        8
#define BASEINFO_MAXRNUM        9
#define BASEINFO_OWNER          10
#define BASEINFO_GRAPH          11
#define BASEINFO_CLUSTERBASES   12
#define BASEINFO_DBNUMBER       13

/* Manifest constants for use with TdbFieldInfo() */
#define FIELDINFO_SIZE          1
#define FIELDINFO_TYPE          2

/* Manifest constants for use with TdbRecordInfo() */
#define RECORDINFO_RID          1
#define RECORDINFO_DBNAME       2
#define RECORDINFO_RNAME        3
#define RECORDINFO_TSTAMP_CODED 4
#define RECORDINFO_TSTAMP_ASCII 5
#define RECORDINFO_CLASS_NAME   6
#define RECORDINFO_SIZE         7
#define RECORDINFO_HITS         8

/* Manifest constants for use with TdbCursorInfo() */
#define CURSORINFO_DB           1
#define CURSORINFO_RID          2
#define CURSORINFO_PARTID       3
#define CURSORINFO_FIELDNR      4
#define CURSORINFO_PARANR       5
#define CURSORINFO_ITEMNR       6
#define CURSORINFO_HITS         7

/* Manifest constants for use with TdbSessionInfo() */
#define SESSIONINFO_RUNTIME         1
#define SESSIONINFO_DATEFORM        2
#define SESSIONINFO_OPENBASES       5
#define SESSIONINFO_CURRENTBASE     6
#define SESSIONINFO_CLUSTER         7
#define SESSIONINFO_DBS_IN_SEARCH   8
#define SESSIONINFO_SIF_NAME        9
#define SESSIONINFO_CHARSET        10
#define SESSIONINFO_SESSION_STATUS 11
#define SESSIONINFO_CJK_MODE       12
#define SESSIONINFO_SUPERMAN       13
#define SESSIONINFO_COMMERCIAL     14
#define SESSIONINFO_DBS_INITIATED  15
#define SESSIONINFO_DB_IN_SEARCH   16
#define SESSIONINFO_HIGHEST_SEARCH 17

/* Manifest constants for use with TdbUserInfo() */
#define USERINFO_FM             1
#define USERINFO_UM             2
#define USERINFO_SM             3
#define USERINFO_STARTPROC      4

/* Constants for comforter mode */

#define COMFORT_SEARCH  1
#define COMFORT_SORT    2
#define COMFORT_WAIT    3

/* Constants for use with the graph API functions. */
#define GRAPH_DIRECTION_FOLLOW 1
#define GRAPH_DIRECTION_BACK   2
#define GRAPH_RESOLVE_SOURCE 1
#define GRAPH_RESOLVE_TARGET 2
#define GRAPH_SET_UNIVERSAL  0
#define GRAPH_PATH_ALL       1
#define GRAPH_PATH_SHORTEST  2
#define GRAPH_SETTYPE_REGULAR 0
#define GRAPH_SETTYPE_GRAPH   1


/* Macros for 'infotype' argument to TdbGetBuildInfo() */
#define BUILDINFO_VERSION_MAJOR    1
#define BUILDINFO_VERSION_MINOR    2
#define BUILDINFO_VERSION_SERVICE  3
#define BUILDINFO_VERSION_PATCH    4
#define BUILDINFO_BUILD_NUMBER     5
#define BUILDINFO_SERVER_PID       6
#define BUILDINFO_SOURCE_REVISION  7

/* Structures and constants related to referential integrity */
#define INTEGRITY_RESTRICT              1
#define INTEGRITY_CASCADE               2
#define INTEGRITY_NOACTION              3
#define INTEGRITY_SETNULL               4
#define INTEGRITY_SETDEFAULT            5

/* Modes for opening containers with TdbSetClassificationContainer() */
#define MODE_VIEW     1 /* Read mode, view classes, etc. */
#define MODE_TRAIN    2 /* Training mode */
#define MODE_CLASSIFY 4 /* Classification mode */


/* Generic macros for testing success / failure of TRIPapi routines */
#if !defined(IsError)
#   define IsError(x)           ( !( (x) & 1) )
#endif

#if !defined(IsSuccess)
#   define IsSuccess(x)         ( (x) & 1 )
#endif


/* Macros for interpreting flags word from TdbCheckField() */
#define IsIndexed(x)            ( (x) & 1 )
#define IsWriteable(x)          ( (x) & 2 )
#define IsReadable(x)           ( (x) & 64 )
#define HasAccess(x)            ( (x) & (64+2))
#define IsPartField(x)          ( (x) & 4 )
#define IsMandatory(x)          ( (x) & 16 )
#define IsLayoutRetained(x)     ( (x) & 32 )

/* Macros for interpreting the return code from TdbGetRecordInBase(),
 * TdbGetRecordInSearch() and TdbGetRecordInSearchRis()
 */
#define IsNewBase(x)            ( (x) & 2 )
#define IsDeleted(x)            ( (x) & 4 )
#define IsLocked(x)             ( (x) & 8 )
#define IsAtEnd(x)              ( (x) & 16 )

/* Macros for interpreting flags word from TdbGetWindowInfo() */
#define IsAtTop(x)              ( (x) & 1 )     /* Top of output */
#define IsAtBottom(x)           ( (x) & 2 )     /* Bottom of output */
#define IsWithFF(x)             ( (x) & 4 )     /* Output format uses <FF> */


#endif

